@@ -28,11 +28,15 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 """
+from __future__ import print_function
+
+from builtins import str
 
 import copy
 import importlib
 import logging
 import os
+import re
 import shutil
 import sys
 import types
@@ -40,6 +44,7 @@ import functools
 import platform
 import argparse
 import hashlib
+import requests
 
 from mslib import __version__
 from mslib.msui.mss_qt import ui_mainwindow as ui
@@ -52,18 +57,28 @@ from mslib.msui import timeseriesview
 from mslib.msui import trajectories_tool
 from mslib.msui import loopview
 from mslib.msui import constants
-from mslib.utils import config_loader
+from mslib.utils import config_loader, setup_logging
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 from mslib.plugins.io.csv import load_from_csv, save_to_csv
-from mslib.msui.icons import icons
+from mslib.msui.icons import icons, python_powered
 
 # related third party imports
 from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, _translate, _fromUtf8, USE_PYQT5
 
+try:
+    import nappy
+    HAVE_NAPPY = True
+except ImportError:
+    HAVE_NAPPY = False
+
 # Add config path to PYTHONPATH so plugins located there may be found
 sys.path.append(constants.MSS_CONFIG_PATH)
 
 
+def clean_string(string):
+    return re.sub('\W|^(?=\d)', '_', string)
+
+
 class QActiveViewsListWidgetItem(QtWidgets.QListWidgetItem):
     """Subclass of QListWidgetItem, represents an open view in the list of
        open views. Keeps a reference to the view instance (i.e. the window) it
@@ -130,7 +145,7 @@ class QFlightTrackListWidgetItem(QtWidgets.QListWidgetItem):
     def nameChanged(self, item):
         """Slot to change the name of a flight track.
         """
-        item.flighttrack_model.setName(unicode(item.text()))
+        item.flighttrack_model.setName(str(item.text()))
 
 
 class MSS_AboutDialog(QtWidgets.QDialog, ui_ab.Ui_AboutMSUIDialog):
@@ -146,6 +161,8 @@ class MSS_AboutDialog(QtWidgets.QDialog, ui_ab.Ui_AboutMSUIDialog):
         super(MSS_AboutDialog, self).__init__(parent)
         self.setupUi(self)
         self.lblVersion.setText(u"Version: {}".format(__version__))
+        blub = QtGui.QPixmap(python_powered())
+        self.lblPython.setPixmap(blub)
 
 
 class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
@@ -165,7 +182,8 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             import ctypes
             myappid = u"mss.mss_pyui.{}".format(__version__)  # arbitrary string
             ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
-        except (ImportError, AttributeError):
+        except (ImportError, AttributeError) as error:
+            logging.debug("AttributeError, ImportError Exception %s", error)
             pass
         # Reference to the flight track that is currently displayed in the
         # views.
@@ -212,51 +230,70 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         self.addImportFilter("CSV", "csv", load_from_csv)
         self.addExportFilter("CSV", "csv", save_to_csv)
 
-        import_plugins = config_loader(dataset="import_plugins", default={})
-        for name in import_plugins:
-            extension, module, function = import_plugins[name]
+        self._imported_plugins, self._exported_plugins = {}, {}
+        self.addPlugins()
+        # self.actionLoopView.setVisible(config_loader(dataset="enable_loopview", default=False))
+
+    def addPlugins(self):
+        self._imported_plugins = config_loader(dataset="import_plugins", default={})
+        for name in self._imported_plugins:
+            extension, module, function = self._imported_plugins[name]
             try:
                 imported_module = importlib.import_module(module)
-            except Exception, ex:
+            except Exception as ex:
                 QtWidgets.QMessageBox.critical(
                     self, self.tr("file io plugin error import plugins"),
                     self.tr(u"ERROR: Configuration\n\n{}\n\nthrows {} error:\n{}".format(
-                        import_plugins, type(ex), ex)))
+                        self._imported_plugins, type(ex), ex)))
                 continue
             try:
                 self.addImportFilter(name, extension, getattr(imported_module, function))
-            except AttributeError, ex:
+            except AttributeError as ex:
                 QtWidgets.QMessageBox.critical(
                     self, self.tr("file io plugin error import plugins"),
                     self.tr(u"ERROR: Configuration\n\n{}\n\nthrows {} error:\n{}".format(
-                        import_plugins, type(ex), ex)))
+                        self._imported_plugins, type(ex), ex)))
                 continue
 
-        export_plugins = config_loader(dataset="export_plugins", default={})
-        for name in export_plugins:
-            extension, module, function = export_plugins[name]
+        self._exported_plugins = config_loader(dataset="export_plugins", default={})
+        for name in self._exported_plugins:
+            extension, module, function = self._exported_plugins[name]
             try:
                 imported_module = importlib.import_module(module)
-            except Exception, ex:
+            except Exception as ex:
                 QtWidgets.QMessageBox.critical(
                     self, self.tr("file io plugin error import plugins"),
                     self.tr(u"ERROR: Configuration\n\n{}\n\nthrows {} error:\n{}".format(
-                        import_plugins, type(ex), ex)))
+                        self._exported_plugins, type(ex), ex)))
                 continue
             try:
                     self.addExportFilter(name, extension, getattr(imported_module, function))
-            except Exception, ex:
+            except Exception as ex:
                 QtWidgets.QMessageBox.critical(
                     self, self.tr("file io plugin error"),
                     self.tr(u"ERROR: Configuration for export {} plugins\n\n{}\n\nthrows error:\n{}".format(
-                        export_plugins, type(ex), ex)))
+                        self._exported_plugins, type(ex), ex)))
                 continue
 
-        # self.actionLoopView.setVisible(config_loader(dataset="enable_loopview", default=False))
+    def removePlugins(self):
+        for name in self._imported_plugins:
+            full_name = "actionImportFlightTrack" + clean_string(name)
+            actions = [_x for _x in self.menuImport_Flight_Track.actions()
+                       if _x.objectName() == _fromUtf8(full_name)]
+            assert len(actions) == 1
+            self.menuImport_Flight_Track.removeAction(actions[0])
+            delattr(self, full_name)
+
+        for name in self._exported_plugins:
+            full_name = "actionExportFlightTrack" + clean_string(name)
+            actions = [_x for _x in self.menuExport_Active_Flight_Track.actions()
+                       if _x.objectName() == _fromUtf8(full_name)]
+            assert len(actions) == 1
+            self.menuExport_Active_Flight_Track.removeAction(actions[0])
+            delattr(self, full_name)
 
     def addImportFilter(self, name, extension, function):
-        full_name = "actionImportFlightTrack" + name.replace(" ", "")
-
+        full_name = "actionImportFlightTrack" + clean_string(name)
         if hasattr(self, full_name):
             raise ValueError(u"'{}' has already been set!".format(full_name))
 
@@ -268,12 +305,11 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         def load_function_wrapper(self):
             filename = QtWidgets.QFileDialog.getOpenFileName(
                 self, "Import Flight Track", "", name + " (*." + extension + ")")
-            filename = filename[0] if USE_PYQT5 else unicode(filename)
-
+            filename = filename[0] if isinstance(filename, list) and USE_PYQT5 else str(filename)
             if filename:
                 try:
                     ft_name, new_waypoints = function(filename)
-                except (SyntaxError, IndexError, OSError, IOError), ex:
+                except (SyntaxError, IndexError, OSError, IOError) as ex:
                     QtWidgets.QMessageBox.critical(
                         self, self.tr("file io plugin error"),
                         self.tr(u"ERROR: {} {}".format(type(ex), ex)))
@@ -292,8 +328,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         action.triggered.connect(getattr(self, full_name))
 
     def addExportFilter(self, name, extension, function):
-        full_name = "actionExportFlightTrack" + name.replace(" ", "")
-
+        full_name = "actionExportFlightTrack" + clean_string(name)
         if hasattr(self, full_name):
             raise ValueError(u"'{}' has already been set!".format(full_name))
 
@@ -305,12 +340,11 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         def save_function_wrapper(self):
             filename = QtWidgets.QFileDialog.getSaveFileName(
                 self, "Export Flight Track", "", name + " (*." + extension + ")")
-            filename = filename[0] if USE_PYQT5 else unicode(filename)
-
+            filename = filename[0] if isinstance(filename, list) and USE_PYQT5 else str(filename)
             if filename:
                 try:
                     function(filename, self.active_flight_track.name, self.active_flight_track.waypoints)
-                except (OSError, IOError), ex:
+                except (OSError, IOError) as ex:
                     QtWidgets.QMessageBox.critical(
                         self, self.tr("file io plugin error"),
                         self.tr(u"ERROR: {} {}".format(type(ex), ex)))
@@ -345,16 +379,24 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
            a new instance of the view and adds a QActiveViewsListWidgetItem to
            the list of open views (self.listViews).
         """
+        layout = config_loader(dataset="layout", default=mss_default.layout)
         view_window = None
         if self.sender() == self.actionTopView:
             # Top view.
             view_window = topview.MSSTopViewWindow(model=self.active_flight_track)
+            view_window.mpl.resize(layout['topview'][0], layout['topview'][1])
+            if layout["immutable"]:
+                view_window.mpl.setFixedSize(layout['topview'][0], layout['topview'][1])
         elif self.sender() == self.actionSideView:
             # Side view.
             view_window = sideview.MSSSideViewWindow(model=self.active_flight_track)
+            view_window.mpl.resize(layout['sideview'][0], layout['sideview'][1])
+            if layout["immutable"]:
+                view_window.mpl.setFixedSize(layout['sideview'][0], layout['sideview'][1])
         elif self.sender() == self.actionTableView:
             # Table view.
             view_window = tableview.MSSTableViewWindow(model=self.active_flight_track)
+            view_window.centralwidget.resize(layout['tableview'][0], layout['tableview'][1])
         if view_window is not None:
             # Make sure view window will be deleted after being closed, not
             # just hidden (cf. Chapter 5 in PyQt4).
@@ -462,11 +504,20 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         Returns:
 
         """
-        filename = QtWidgets.QFileDialog.getOpenFileName(
-            self, "Open Config file", "", "Supported files (*.json *.txt)")
-        filename = filename[0] if USE_PYQT5 else unicode(filename)
-        if filename:
-            constants.CACHED_CONFIG_FILE = filename
+        ret = QtWidgets.QMessageBox.warning(
+            self, self.tr("Mission Support System"),
+            self.tr("Opening a config file will reset application. Continue?"),
+            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
+        if ret == QtWidgets.QMessageBox.Yes:
+            filename = QtWidgets.QFileDialog.getOpenFileName(
+                self, "Open Config file", "", "Supported files (*.json *.txt)")
+            filename = filename[0] if isinstance(filename, list) and USE_PYQT5 else str(filename)
+            if filename:
+                self.listViews.clear()
+                self.listTools.clear()
+                self.removePlugins()
+                constants.CACHED_CONFIG_FILE = filename
+                self.addPlugins()
 
     def openFlightTrack(self):
         """Slot for the 'Open Flight Track' menu entry. Opens a QFileDialog and
@@ -475,8 +526,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         filename = QtWidgets.QFileDialog.getOpenFileName(self,
                                                          "Open Flight Track", "",
                                                          "Flight track XML (*.ftml)")
-        filename = filename[0] if USE_PYQT5 else unicode(filename)
-
+        filename = filename[0] if isinstance(filename, list) and USE_PYQT5 else str(filename)
         if filename:
             if filename.endswith('.ftml'):
                 self.createNewFlightTrack(filename=filename, activate=True)
@@ -526,8 +576,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         default_filename = os.path.join(self.lastSaveDir, self.active_flight_track.name + ".ftml")
         filename = QtWidgets.QFileDialog.getSaveFileName(
             self, "Save Flight Track", default_filename, "Flight track XML (*.ftml)")
-        filename = filename[0] if USE_PYQT5 else unicode(filename)
-
+        filename = filename[0] if isinstance(filename, list) and USE_PYQT5 else str(filename)
         if filename:
             self.lastSaveDir = os.path.dirname(filename)
             if filename.endswith('.ftml'):
@@ -572,41 +621,22 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         dlg.setModal(True)
         dlg.exec_()
 
-
-def setup_logging(args):
-    logger = logging.getLogger()
-    # this is necessary as "someone" has already initialized logging, preventing basicConfig from doing stuff
-    for ch in logger.handlers:
-        logger.removeHandler(ch)
-
-    debug_formatter = logging.Formatter("%(asctime)s (%(module)s.%(funcName)s:%(lineno)s): %(message)s")
-    default_formatter = logging.Formatter("%(levelname)s: %(message)s")
-
-    # Console handler (suppress DEBUG by default)
-    ch = logging.StreamHandler()
-    if args.debug:
-        logger.setLevel(logging.DEBUG)
-        ch.setLevel(logging.DEBUG)
-        ch.setFormatter(debug_formatter)
-    else:
-        logger.setLevel(logging.INFO)
-        ch.setLevel(logging.INFO)
-        ch.setFormatter(default_formatter)
-    logger.addHandler(ch)
-    # File handler (always on DEBUG level)
-    if not args.nolog:
-        logfile = os.path.join(constants.MSS_CONFIG_PATH, "mss_pyui.log")
-        if args.logfile is not None:
-            logfile = args.logfile
+    def configureMenu(self):
+        """
+        disables menu entries which won't work, because libraries missing or not configured
+        """
+        # loopview
         try:
-            fh = logging.FileHandler(logfile, "w")
-        except (OSError, IOError), ex:
-            logger.error("Could not open logfile '%s': %s %s", logfile, type(ex), ex)
-        else:
-            logger.setLevel(logging.DEBUG)
-            fh.setLevel(logging.DEBUG)
-            fh.setFormatter(debug_formatter)
-            logger.addHandler(fh)
+            loop_configuration = config_loader(dataset="loop_configuration", default=mss_default.loop_configuration)
+            url = loop_configuration["ECMWF forecasts"]["url"]
+            requests.head(url)
+        except (KeyError, requests.exceptions.ConnectionError):
+            self.actionLoopView.setEnabled(False)
+
+        # trajectory analyses
+        if not HAVE_NAPPY:
+            self.actionTrajectoryToolLagranto.setEnabled(False)
+            self.actionTimeSeriesViewTrajectories.setEnabled(False)
 
 
 def main():
@@ -617,24 +647,24 @@ def main():
     app_prefix = prefix
     if prefix:
         app_prefix = "-{}".format(prefix)
-    icon_hash = hashlib.md5('.'.join([__version__, app_prefix])).hexdigest()
+    icon_hash = hashlib.md5('.'.join([__version__, app_prefix]).encode('utf-8')).hexdigest()
 
     parser = argparse.ArgumentParser()
     parser.add_argument("-v", "--version", help="show version", action="store_true", default=False)
     parser.add_argument("--debug", help="show debugging log messages on console", action="store_true", default=False)
-    parser.add_argument("--logfile", help="specify logfile location", action="store", default=None)
-    parser.add_argument("--nolog", help="do write debug log", action="store_true", default=False)
+    parser.add_argument("--logfile", help="Specify logfile location. Set to empty string to disable.", action="store",
+                        default=os.path.join(constants.MSS_CONFIG_PATH, "mss_pyui.log"))
     parser.add_argument("-m", "--menu", help="adds mss to menu", action="store_true", default=False)
     parser.add_argument("-d", "--deinstall", help="removes mss from menu", action="store_true", default=False)
 
     args = parser.parse_args()
 
     if args.version:
-        print "***********************************************************************"
-        print "\n            Mission Support System (mss)\n"
-        print "***********************************************************************"
-        print "Documentation: http://mss.rtfd.io"
-        print "Version:", __version__
+        print("***********************************************************************")
+        print("\n            Mission Support System (mss)\n")
+        print("***********************************************************************")
+        print("Documentation: http://mss.rtfd.io")
+        print("Version:", __version__)
         sys.exit()
 
     if args.menu:
@@ -675,6 +705,7 @@ def main():
     logging.info("Launching user interface...")
     application = QtWidgets.QApplication(sys.argv)
     mainwindow = MSSMainWindow()
+    mainwindow.configureMenu()
     mainwindow.createNewFlightTrack(activate=True)
     mainwindow.show()
     sys.exit(application.exec_())
