@@ -26,6 +26,8 @@
     limitations under the License.
 """
 
+from builtins import str
+
 import functools
 from mslib.utils import config_loader, save_settings_pickle, load_settings_pickle
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
@@ -149,8 +151,9 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
            values are corrected.
         """
         try:
-            flightlevel = int(float(unicode(item.text())))
-        except:
+            flightlevel = int(float(str(item.text())))
+        except Exception as ex:
+            logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
             flightlevel = 0
         if flightlevel < 0:
             flightlevel = 0
@@ -162,7 +165,7 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
     def getFlightLevels(self):
         """Returns the flight level values contained in the table.
         """
-        return [int(unicode(self.tableWidget.item(row, 0).text()))
+        return [int(str(self.tableWidget.item(row, 0).text()))
                 for row in range(self.tableWidget.rowCount())]
 
     def getSettings(self):
