@@ -26,36 +26,50 @@
     limitations under the License.
 """
 
+from future import standard_library
+standard_library.install_aliases()
+
+from past.builtins import basestring
+from builtins import str
+
 import time
 from datetime import datetime, timedelta
 
-import StringIO
+import io
 import hashlib
 import logging
 import os
 import requests
 import re
-import urllib
-import urllib2
+import urllib.parse
 import xml.etree.ElementTree as etree
 from mslib.utils import config_loader
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
-# related third party imports
-from mslib.msui.mss_qt import QtCore, QtWidgets, USE_PYQT5
+from mslib.msui.mss_qt import QtCore, QtGui, QtWidgets, USE_PYQT5
 
-import mslib.owslib.wms
-import mslib.owslib.util
+import mslib.ogcwms
+import owslib.util
 import PIL.Image
 
-# local application imports
 from mslib.msui.mss_qt import ui_wms_dockwidget as ui
 from mslib.msui.mss_qt import ui_wms_password_dialog as ui_pw
 from mslib.msui import wms_capabilities
 from mslib.msui import constants
 from mslib.utils import convertHPAToKM
+from mslib.ogcwms import openURL
 
 
-class MSSWebMapService(mslib.owslib.wms.WebMapService):
+WMS_SERVICE_CACHE = {}
+WMS_URL_LIST = QtGui.QStandardItemModel()
+
+
+def add_wms_urls(combo_box, url_list):
+    combo_box_urls = [str(combo_box.itemText(_i)) for _i in range(combo_box.count())]
+    for url in (_url for _url in url_list if _url not in combo_box_urls):
+        combo_box.addItem(url)
+
+
+class MSSWebMapService(mslib.ogcwms.WebMapService):
     """Overloads the getmap() method of owslib.wms.WebMapService:
 
         added parameters are
@@ -167,28 +181,34 @@ class MSSWebMapService(mslib.owslib.wms.WebMapService):
 
         if level is not None:
             request['elevation'] = str(level)
+
+        # normalise base_url so it contains no request and no parameters
+        (scheme, netloc, path, params, query, fragment) = urllib.parse.urlparse(base_url)
+        base_url = urllib.parse.urlunparse((scheme, netloc, path, params, "", fragment))
+        request.update(dict(urllib.parse.parse_qsl(query)))
         # --(mss)
 
-        data = urllib.urlencode(request)
+        data = urllib.parse.urlencode(request)
 
         # ++(mss)
         base_url = base_url.replace("ogctest.iblsoft", "ogcie.iblsoft")  # IBL Bugfix!
         base_url = base_url.replace("ogcie/obs", "ogcie.iblsoft.com/obs")  # IBL Bugfix!
         base_url = base_url.replace(", staging1", "")  # geo.beopen.eu bugfix
-        complete_url = u"{}?{}".format(base_url, data)
+
+        complete_url = urllib.parse.urlunparse((str(""), str(""), str(base_url), str(""), data, str("")))
         if return_only_url:
             return complete_url
-        logging.debug("Retrieving: %s", complete_url)
+        logging.debug("Retrieving image from '%s'", complete_url)
         # --(mss)
 
         # (mss) owslib.util.openURL checks for ServiceExceptions and raises a
         # owslib.wms.ServiceException. However, openURL only checks for mime
         # types text/xml and application/xml. application/vnd.ogc.se_xml is
         # not considered. For some reason, the check below doesn't work, though..
+        proxies = config_loader(dataset="proxies", default=mss_default.proxies)
 
-        u = mslib.owslib.util.openURL(base_url, data, method,
-                                      username=self.username,
-                                      password=self.password)
+        u = openURL(base_url, data, method,
+                    username=self.username, password=self.password, proxies=proxies)
 
         # check for service exceptions, and return
         # NOTE: There is little bug in owslib.util.openURL -- if the file
@@ -200,12 +220,14 @@ class MSSWebMapService(mslib.owslib.wms.WebMapService):
         if hasattr(u, "info") and u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
             se_xml = u.read()
             se_tree = etree.fromstring(se_xml)
-            err_message = unicode(se_tree.find('ServiceException').text).strip()
-            raise mslib.owslib.wms.ServiceException(err_message, se_xml)
+            err_message = str(se_tree.find('ServiceException').text).strip()
+            raise owslib.wms.ServiceException(err_message, se_xml)
         return u
 
     def get_redirect_url(self, method="Get"):
-        return self.getOperationByName("GetMap").methods[method]["url"]
+        # return self.getOperationByName("GetMap").methods[method]["url"]
+        # ToDo redirect broken
+        return self.getOperationByName("GetMap").methods[0]["url"]
 
 
 class MSS_WMS_AuthenticationDialog(QtWidgets.QDialog, ui_pw.Ui_WMSAuthenticationDialog):
@@ -224,8 +246,8 @@ class MSS_WMS_AuthenticationDialog(QtWidgets.QDialog, ui_pw.Ui_WMSAuthentication
     def getAuthInfo(self):
         """Return the entered username and password.
         """
-        return (unicode(self.leUsername.text()),
-                unicode(self.lePassword.text()))
+        return (str(self.leUsername.text()),
+                str(self.lePassword.text()))
 
 
 class WMSMapFetcher(QtCore.QObject):
@@ -276,7 +298,7 @@ class WMSMapFetcher(QtCore.QObject):
         try:
             map_img = self.fetch_map(kwargs, use_cache, md5_filename)
             legend_img = self.fetch_legend(use_cache=use_cache, **legend_kwargs)
-        except Exception, ex:
+        except Exception as ex:
             logging.error("MapPrefetcher Exception %s - %s.", type(ex), ex)
             # emit finished so progress dialog will be closed
             self.finished.emit(None, None, None, None, None, None, md5_filename)
@@ -302,11 +324,11 @@ class WMSMapFetcher(QtCore.QObject):
             self.started_request.emit()
             self.long_request = True
             urlobject = self.wms.getmap(**kwargs)
-            image_io = StringIO.StringIO(urlobject.read())
+            image_io = io.BytesIO(urlobject.read())
             img = PIL.Image.open(image_io)
             # Check if the image is stored as indexed palette
             # with a transparent colour. Store correspondingly.
-            if img.mode == "P" and "transparency" in img.info.keys():
+            if img.mode == "P" and "transparency" in img.info:
                 img.save(md5_filename, transparency=img.info["transparency"])
             else:
                 img.save(md5_filename)
@@ -332,15 +354,15 @@ class WMSMapFetcher(QtCore.QObject):
             # PIL.Image.open(). See
             #    http://www.pythonware.com/library/pil/handbook/image.htm
             logging.debug("Retrieving legend from '%s'", urlstr)
-            urlobject = urllib2.urlopen(urlstr)
-            image_io = StringIO.StringIO(urlobject.read())
+            urlobject = requests.get(urlstr)
+            image_io = io.BytesIO(urlobject.content)
             legend_img_raw = PIL.Image.open(image_io)
             legend_img = legend_img_raw.crop(legend_img_raw.getbbox())
-            logging.debug("legend retrieved, legend graphic size is %i bytes.", image_io.len)
             # Store the retrieved image in the cache, if enabled.
             try:
                 legend_img.save(md5_filename, transparency=0)
-            except:
+            except Exception as ex:
+                logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
                 legend_img.save(md5_filename)
         return legend_img
 
@@ -375,12 +397,14 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
         # Accomodates MSSWebMapService instances.
         self.wms = None
-        self.wms_service_cache = {}
 
         # Initial list of WMS servers.
-        self.cbWMS_URL.clear()
+        self.cbWMS_URL.setModel(WMS_URL_LIST)
+
         if default_WMS is not None:
-            self.cbWMS_URL.addItems(default_WMS)
+            add_wms_urls(self.cbWMS_URL, default_WMS)
+        if self.cbWMS_URL.count() > 0:
+            self.cbWMS_URL.setCurrentIndex(0)
 
         # Compile regular expression used in crsAllowed() to filter
         # layers accordings to their CRS.
@@ -526,18 +550,18 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         """
         wms = None
         # initialize login cache fomr config file, but do not overwrite existing keys
-        for key, value in config_loader(dataset="WMS_login", default={}).items():
+        for key, value in list(config_loader(dataset="WMS_login", default={}).items()):
             if key not in constants.WMS_LOGIN_CACHE:
                 constants.WMS_LOGIN_CACHE[key] = value
         username, password = constants.WMS_LOGIN_CACHE.get(base_url, (None, None))
 
         try:
-            _ = str(base_url)  # to provoke early Unicode Error
+            str(base_url)  # to provoke early Unicode Error
             while wms is None:
                 try:
                     wms = MSSWebMapService(base_url, version='1.1.1',
                                            username=username, password=password)
-                except mslib.owslib.util.ServiceException as ex:
+                except owslib.util.ServiceException as ex:
                     if str(ex).startswith("401") or str(ex).find("Error 401") >= 0 or str(ex).find(
                             "401 Unauthorized") >= 0:
                         # Catch the "401 Unauthorized" error if one has been
@@ -570,12 +594,14 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         return wms
 
     def wmsUrlChanged(self, text):
-        text = unicode(text)
-        wms = self.wms_service_cache.get(text)
+        text = str(text)
+        wms = WMS_SERVICE_CACHE.get(text)
         if wms is not None and wms != self.wms:
             self.activateWMS(wms)
         elif self.wms is not None:
             self.wms = None
+            self.btGetMap.setEnabled(False)
+
             self.cbLayer.clear()
             self.cbLevel.clear()
             self.cbStyle.clear()
@@ -587,7 +613,6 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
             self.enableLevelElements(False)
             self.enableValidTimeElements(False)
             self.enableInitTimeElements(False)
-            self.btGetMap.setEnabled(False)
             self.pbViewCapabilities.setEnabled(False)
             self.cbTransparent.setChecked(False)
 
@@ -613,42 +638,53 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Load new WMS. Only add those layers to the combobox that can provide
         # the CRS that match the filter of this module.
 
-        base_url = unicode(self.cbWMS_URL.currentText())
+        base_url = str(self.cbWMS_URL.currentText())
         try:
             request = requests.get(base_url)
-        except requests.exceptions.ConnectionError:
-            request = None
+        except (requests.exceptions.TooManyRedirects,
+                requests.exceptions.ConnectionError,
+                requests.exceptions.InvalidURL,
+                requests.exceptions.InvalidSchema,
+                requests.exceptions.MissingSchema) as ex:
+            logging.error("Cannot load capabilities document.\n"
+                          "No layers can be used in this view.")
+            QtWidgets.QMessageBox.critical(
+                self, self.tr("Web Map Service"),
+                self.tr(u"ERROR: We cannot load the capability document!\n\n{}\n{}".format(type(ex), ex)))
+        else:
+            logging.debug(u"requesting capabilities from %s", request.url)
+            wms = self.initialiseWMS(request.url)
+            if wms is not None:
 
-        if request is not None and request.status_code == 200 and request.url != base_url:
-            found = False
-            for count in range(self.cbWMS_URL.count()):
-                if self.cbWMS_URL.itemText(count) == base_url:
-                    self.cbWMS_URL.setItemText(count, request.url)
-                    self.cbWMS_URL.setCurrentIndex(count)
-                    found = True
-                    break
-                if self.cbWMS_URL.itemText(count) == request.url:
-                    self.cbWMS_URL.setCurrentIndex(count)
-                    found = True
-            if not found:
-                self.cbWMS_URL.insertItem(self.cbWMS_URL.count(), request.url)
-                self.cbWMS_URL.setCurrentIndex(self.cbWMS_URL.count() - 1)
-            base_url = request.url
-        logging.debug(u"requesting capabilities from %s", base_url)
-        wms = self.initialiseWMS(base_url)
-        if wms is not None:
-            self.activateWMS(wms)
-            self.wms_service_cache[wms.url] = wms
+                # update the combo box, if entry requires change/insertion
+                found = False
+                for count in range(self.cbWMS_URL.count()):
+                    if self.cbWMS_URL.itemText(count) == base_url:
+                        self.cbWMS_URL.setItemText(count, request.url)
+                        self.cbWMS_URL.setCurrentIndex(count)
+                        found = True
+                        break
+                    if self.cbWMS_URL.itemText(count) == request.url:
+                        self.cbWMS_URL.setCurrentIndex(count)
+                        found = True
+                if not found:
+                    logging.debug("inserting URL: %s", request.url)
+                    add_wms_urls(self.cbWMS_URL, [request.url])
+                    self.cbWMS_URL.setEditText(request.url)
+
+                self.activateWMS(wms)
+                WMS_SERVICE_CACHE[wms.url] = wms
 
     def activateWMS(self, wms):
         # Clear layer and style combo boxes. First disconnect the layerChanged
         # slot to avoid calls to this function.
+        self.btGetMap.setEnabled(False)
         self.cbLayer.currentIndexChanged.disconnect(self.layerChanged)
         self.cbLayer.clear()
         self.cbStyle.clear()
 
         # Parse layer tree of the wms object and discover usable layers.
-        stack = wms.contents.values()
+        stack = list(wms.contents.values())
         filtered_layers = []
         while len(stack) > 0:
             layer = stack.pop()
@@ -666,7 +702,6 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         self.cbLayer.setEnabled(self.cbLayer.count() > 1)
         self.wms = wms
         self.layerChanged(0)
-        self.btGetMap.setEnabled(True)
         self.pbViewCapabilities.setEnabled(True)
 
         if self.prefetcher is not None:
@@ -694,6 +729,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
         # Reconnect layerChanged.
         self.cbLayer.currentIndexChanged.connect(self.layerChanged)
+        self.btGetMap.setEnabled(True)
 
     def viewCapabilities(self):
         """Open a WMSCapabilitiesBrowser dialog showing the capabilities
@@ -740,7 +776,8 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
                     return format
                 else:
                     return d
-            except ValueError:
+            except ValueError as error:
+                logging.debug("ValueError Exception %s", error)
                 pass
         return None
 
@@ -750,10 +787,10 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         """
         if self.wms is None:
             return None
-        if layername in self.wms.contents.keys():
+        if layername in self.wms.contents:
             return self.wms.contents[layername]
         else:
-            stack = self.wms.contents.values()
+            stack = list(self.wms.contents.values())
             while len(stack) > 0:
                 layer = stack.pop()
                 if layer.name == layername:
@@ -797,7 +834,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # ~~~~ A) Elevation. ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         lobj = layerobj
         while lobj is not None:
-            if "elevation" in lobj.dimensions.keys() and "elevation" in lobj.extents.keys():
+            if "elevation" in lobj.dimensions and "elevation" in lobj.extents:
                 units = lobj.dimensions["elevation"]["units"]
                 elev_list = [u"{} ({})".format(e.strip(), units) for e in
                              lobj.extents["elevation"]["values"]]
@@ -816,11 +853,11 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
         lobj = layerobj
         while lobj is not None:
-            if "init_time" in lobj.dimensions.keys() and "init_time" in lobj.extents.keys():
+            if "init_time" in lobj.dimensions and "init_time" in lobj.extents:
                 # MSS web map service.
                 self.init_time_name = "init_time"
                 enable_inittime = True
-            elif "run" in lobj.dimensions.keys() and "run" in lobj.extents.keys():
+            elif "run" in lobj.dimensions and "run" in lobj.extents:
                 # IBL web map service.
                 self.init_time_name = "run"
                 enable_inittime = True
@@ -856,14 +893,14 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         lobj = layerobj
         while lobj is not None:
 
-            if "time" in lobj.dimensions.keys() and "time" in lobj.extents.keys():
+            if "time" in lobj.dimensions and "time" in lobj.extents:
                 self.valid_time_name = "time"
                 enable_validtime = True
-            elif "time" in lobj.dimensions.keys():
+            elif "time" in lobj.dimensions:
                 enable_validtime = True
                 self.valid_time_name = "time"
                 vtime_no_extent = True
-            elif "forecast" in lobj.dimensions.keys() and "forecast" in lobj.extents.keys():
+            elif "forecast" in lobj.dimensions and "forecast" in lobj.extents:
                 # IBL web map service.
                 self.valid_time_name = "forecast"
                 enable_validtime = True
@@ -921,8 +958,8 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
                                         self.cbValidTime.addItem(
                                             time_val.strftime(self.valid_time_format))
                                         time_val += timedelta(hours=time_interval_hours)
-
-                                except:
+                                except Exception as ex:
+                                    logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
                                     interpretation_successful = False
 
                             else:
@@ -932,7 +969,8 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
                                         datetime.strptime(time_item,
                                                           self.valid_time_format))
                                     self.cbValidTime.addItem(time_item)
-                                except:
+                                except Exception as ex:
+                                    logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
                                     interpretation_successful = False
 
                             if not interpretation_successful:
@@ -1015,17 +1053,20 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         try:
             minutes = int(timestep_string.split(" min")[0])
             return minutes * 60
-        except ValueError:
+        except ValueError as error:
+            logging.debug("ValueError Exception %s", error)
             pass
         try:
             hours = int(timestep_string.split(" hour")[0])
             return hours * 3600
-        except ValueError:
+        except ValueError as error:
+            logging.debug("ValueError Exception %s", error)
             pass
         try:
             days = int(timestep_string.split(" days")[0])
             return days * 86400
-        except ValueError:
+        except ValueError as error:
+            logging.debug("ValueError Exception %s", error)
             raise ValueError(u"cannot convert '{}' to seconds: wrong format.".format(timestep_string))
 
     def init_time_back_click(self):
@@ -1114,7 +1155,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         performed (lock mechanism to prevent erroneous requests during
         a layer change).
         """
-        if self.cbAutoUpdate.isChecked() and not self.layerChangeInProgress:
+        if self.btGetMap.isEnabled() and self.cbAutoUpdate.isChecked() and not self.layerChangeInProgress:
             self.btGetMap.click()
 
     def check_init_time(self, dt):
@@ -1309,18 +1350,16 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         style = self.getStyle()
         layerobj = self.get_layer_object(layer)
         urlstr = None
-        if style != "" and "legend" in layerobj.styles[style].keys():
+        if style != "" and "legend" in layerobj.styles[style]:
             urlstr = layerobj.styles[style]["legend"]
 
         return urlstr
 
     def getMD5Filename(self, kwargs):
-        kwargs["return_only_url"] = True
-        urlstr = self.wms.getmap(**kwargs)
-        kwargs["return_only_url"] = False
+        urlstr = self.wms.getmap(return_only_url=True, **kwargs)
         if not self.wms.url.startswith(self.cbWMS_URL.currentText()):
             raise RuntimeError("WMS URL does not match, use get capabilities first.")
-        return os.path.join(self.wms_cache, hashlib.md5(urlstr).hexdigest() + ".png")
+        return os.path.join(self.wms_cache, hashlib.md5(urlstr.encode('utf-8')).hexdigest() + ".png")
 
     def retrieveImage(self, crs="EPSG:4326", bbox=None, path_string=None,
                       width=800, height=400):
@@ -1399,7 +1438,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
             legend_kwargs = {"urlstr": self.getLegendURL(), "md5_filename": None}
             if legend_kwargs["urlstr"] is not None:
                 legend_kwargs["md5_filename"] = os.path.join(
-                    self.wms_cache, hashlib.md5(legend_kwargs["urlstr"]).hexdigest() + ".png")
+                    self.wms_cache, hashlib.md5(legend_kwargs["urlstr"].encode('utf-8')).hexdigest() + ".png")
 
             # If caching is enabled, get the URL and check the image cache
             # directory for the suitable image file.
@@ -1421,10 +1460,11 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
                     ci_time, ci_level = self.cbValidTime.currentIndex(), self.cbLevel.currentIndex()
                     prefetch_key_values = \
-                        [("time", unicode(self.cbValidTime.itemText(ci_p)))
-                         for ci_p in range(ci_time + 1, ci_time + 1 + pre_tfwd) + range(ci_time - pre_tbck, ci_time)
+                        [("time", str(self.cbValidTime.itemText(ci_p)))
+                         for ci_p in list(range(ci_time + 1, ci_time + 1 + pre_tfwd)) +
+                            list(range(ci_time - pre_tbck, ci_time))
                          if 0 <= ci_p < self.cbValidTime.count()] + \
-                        [("level", unicode(self.cbLevel.itemText(ci_p), errors="ignore").split(" (")[0])
+                        [("level", str(self.cbLevel.itemText(ci_p)).split(" (")[0])
                          for ci_p in range(ci_level - pre_lbck, ci_level + 1 + pre_lfwd)
                          if ci_p != ci_level and 0 <= ci_p < self.cbLevel.count()]
 
@@ -1452,7 +1492,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         if img is None:
             return
 
-        complete_level = unicode(self.cbLevel.currentText())
+        complete_level = str(self.cbLevel.currentText())
         complete_level = complete_level if complete_level != "" else None
         self.displayRetrievedImage(img, legend_img, layer, style, init_time, valid_time, complete_level)
 
@@ -1627,7 +1667,7 @@ class HSecWMSControlWidget(WMSControlWidget):
             lvl = float(s.split(" (")[0])
             if s.endswith("(hPa)"):
                 lvl = convertHPAToKM(lvl)
-            if self.cbAutoUpdate.isChecked() and not self.layerChangeInProgress:
+            if self.btGetMap.isEnabled() and self.cbAutoUpdate.isChecked() and not self.layerChangeInProgress:
                 self.btGetMap.click()
             else:
                 self.view.waypoints_interactor.update()
