@@ -26,21 +26,23 @@
     limitations under the License.
 """
 
+from __future__ import division
+
+
 from datetime import datetime
 
 import logging
 import os
 from abc import ABCMeta, abstractmethod
 
-# related third party imports
 import numpy as np
 
-# local application imports
 from mslib import netCDF4tools
 from mslib import utils
+from future.utils import with_metaclass
 
 
-class MSSPlotDriver(object):
+class MSSPlotDriver(with_metaclass(ABCMeta, object)):
     """
     Abstract super class for implementing driver classes that provide
     access to the MSS data server.
@@ -59,7 +61,6 @@ class MSSPlotDriver(object):
     Classes that derive from this class need to implement the two methods
     set_plot_parameters() and plot().
     """
-    __metaclass__ = ABCMeta
 
     def __init__(self, data_access_object):
         """Requires an instance of a data access object from the MSS
@@ -106,7 +107,7 @@ class MSSPlotDriver(object):
             logging.error(msg)
             raise ValueError(msg)
         fc_step = fc_time - init_time
-        fc_step = fc_step.days * 24 + fc_step.seconds / 3600
+        fc_step = fc_step.days * 24 + (fc_step.seconds // 3600)
         self.fc_time = fc_time
         logging.debug(u"\trequested initialisation time {}".format(init_time))
         logging.debug(u"\trequested forecast valid time {} (step {} hrs)".format(fc_time, fc_step))
@@ -401,8 +402,8 @@ class VerticalSectionDriver(MSSPlotDriver):
         vsec_path = vsec_path if vsec_path is not None else self.vsec_path
         vsec_numpoints = vsec_numpoints if vsec_numpoints is not None else self.vsec_numpoints
         vsec_numlabels = vsec_numlabels if vsec_numlabels is not None else self.vsec_numlabels
-        vsec_path_connection = vsec_path_connection if vsec_path_connection is not None \
-            else self.vsec_path_connection
+        if vsec_path_connection is None:
+            vsec_path_connection = self.vsec_path_connection
         show = show if show else self.show
         transparent = transparent if transparent is not None else self.transparent
         return_format = return_format if return_format is not None else self.return_format
@@ -427,9 +428,10 @@ class VerticalSectionDriver(MSSPlotDriver):
         """
         logging.debug("computing {:} interpolation points, connection: {}"
                       .format(vsec_numpoints, vsec_path_connection))
-        self.lats, self.lons = utils.path_points(vsec_path,
-                                                 numpoints=vsec_numpoints,
-                                                 connection=vsec_path_connection)
+        now = datetime.now()
+        self.lats, self.lons, _ = utils.path_points(
+            [(_x, _y, now) for _x, _y in vsec_path],
+            numpoints=vsec_numpoints, connection=vsec_path_connection)
         self.vsec_path = vsec_path
         self.vsec_numpoints = vsec_numpoints
         self.vsec_path_connection = vsec_path_connection
@@ -470,20 +472,20 @@ class VerticalSectionDriver(MSSPlotDriver):
         lon_indices = lon_data.argsort()
         lon_data = lon_data[lon_indices]
 
-        for name, var in self.data_vars.items():
+        for name, var in list(self.data_vars.items()):
             if len(var.shape) == 4:
                 var_data = var[timestep, ::-self.vert_order, ::self.lat_order, :]
             else:
                 var_data = var[:][timestep, np.newaxis, ::self.lat_order, :]
             logging.debug("\tLoaded {:.2f} Mbytes from data field <{:}> at timestep {:}.".format(
-                          var_data.nbytes / 1048576., name, timestep))
+                          (var_data.nbytes / 1048576.), name, timestep))
             logging.debug("\tVertical dimension direction is {}.".format(
                           "up" if self.vert_order == 1 else "down"))
             logging.debug("\tInterpolating to cross-section path.")
             # Re-arange longitude dimension in the data field.
             var_data = var_data[:, :, lon_indices]
-            data[name] = utils.interpolate_vertsec3(var_data, self.lat_data, lon_data,
-                                                    self.lats, self.lons)
+            data[name] = utils.interpolate_vertsec(var_data, self.lat_data, lon_data,
+                                                   self.lats, self.lons)
             # Free memory.
             del var_data
 
@@ -511,7 +513,7 @@ class VerticalSectionDriver(MSSPlotDriver):
         self.lons = self.lons[lon_indices]
 
         # Shift data fields correspondingly.
-        for key in self.data.keys():
+        for key in self.data:
             self.data[key] = self.data[key][:, lon_indices]
 
     def plot(self):
@@ -633,7 +635,7 @@ class HorizontalSectionDriver(MSSPlotDriver):
             self.actual_level = self.vert_data[level]
         logging.debug("loading data for time step {} ({}), level index {} (level {})".format(
                       timestep, self.fc_time, level, self.level))
-        for name, var in self.data_vars.items():
+        for name, var in list(self.data_vars.items()):
             if level is None or len(var.shape) == 3:
                 # 2D fields: time, lat, lon.
                 var_data = var[timestep, ::self.lat_order, :]
